%% This program computes the structure constants of the free nilpotene
%% Lie algebra with ngv generators and of degree rgv
  
%% Requirements:
%% The number of generators: ngv
%% The degree: rgv

%% Results:
%% The dimension
%% The structure constants in terms of a Jordan-Holder basis
%% written to a file such that the archive command 'archstr' can be
%% applied

%% Remark:
%% The name of the algebra has to be inserted by hand

%% Here comes the number of generators and the degree:
   ngv:=2;
   rgv:=6;

%% Preparation:
   dgo(1):=1;
   dgo(2):=ngv+1;
   jlv:=ngv+1;
   For slv:=1:ngv do <<firstgo(slv):=0; lastgo(slv):=0>>;

%% The Hall basis is found:
   For rlv:=1:rgv-1 do
   Begin
      For slv:=1:rlv do
       for ulv:=dgo(rlv+1-slv):dgo(rlv+2-slv)-1 do
        for vlv:=dgo(slv):dgo(slv+1)-1 do
         if ulv>vlv and lastgo(ulv) <= vlv then
          << firstgo(jlv):=ulv; lastgo(jlv):=vlv; jlv:=jlv+1>>;
   dgo(rlv+2):=jlv;
   End;

%% We now have the dimension   
   mgv:=jlv-1;

%% The operator brack(.,.) (=[.,.]) is defined in the cases where we
%% now know it, namely on the diagonal and where it defines the Hall basis:
   noncom x;
   for jlv:=1:mgv do brack(jlv,jlv):=0;
   for ulv:=1:mgv do 
    << brack(firstgo(ulv),lastgo(ulv)):=x(ulv);
       brack(lastgo(ulv),firstgo(ulv)):=-x(ulv)
    >>;

%% The brack(.,.) is set equal to 0 where we know it is for reasons of
%% degree: 
   For slv:=1:rgv do
    for ulv:=dgo(slv):dgo(slv+1)-1 do
     for rlv:=rgv+1-slv:rgv do
      for vlv:=dgo(rlv):dgo(rlv+1)-1 do
       brack(ulv,vlv):=0;

%% The commutation relations are now defined:
   For ulv:=1:mgv do for vlv:=1:ulv-1 do
    Let x(ulv)*x(vlv)=x(vlv)*x(ulv)+brack(ulv,vlv);

%% The brack(.,.) is now defined for the rest:
   For rlv:=1:rgv-1 do
   Begin
      For slv:=1:rlv do
       for ulv:=dgo(rlv+1-slv):dgo(rlv+2-slv)-1 do
        for vlv:=dgo(slv):dgo(slv+1)-1 do
         if ulv>vlv and lastgo(ulv) > vlv then
      Begin
         clv:=X(lastgo(ulv))*X(vlv)-X(vlv)*X(lastgo(ulv));
         dlv:=X(firstgo(ulv))*X(vlv)-X(vlv)*X(firstgo(ulv));
         brack(ulv,vlv):=
          X(firstgo(ulv))*clv-clv*X(firstgo(ulv))
           -X(lastgo(ulv))*dlv+dlv*X(lastgo(ulv));
         brack(vlv,ulv):=-brack(ulv,vlv);
         For ulv:=1:mgv do for vlv:=1:ulv-1 do
         Let x(ulv)*x(vlv)=x(vlv)*x(ulv)+brack(ulv,vlv);
      End;  
   End;
 
%% The brackets are written on the screen:
   For ulv:=1:mgv do for vlv:=1:mgv do if brack(ulv,vlv) neq 0 then
    write "[X(",ulv,"),X(",vlv,")]=",brack(ulv,vlv);

%% The structure constants are computed (in reverse order):
   array c(mgv,mgv,mgv); 
   For ulv:=1:mgv do for vlv:=1:ulv-1 do if brack(mgv+1-vlv,mgv+1-ulv)
    neq 0 then  for rlv:=1:mgv do  c(ulv,vlv,rlv):=
     -df(brack(mgv+1-vlv,mgv+1-ulv),x(mgv+1-rlv));

%% The results are written to file:
   out "$HOME/REDUCE/LIE/PROGS/cnew";

   off echo;
   write "%% free nilpotent with ",ngv," generators and degree ",r;
   write "Name:=n",mgv,"n?;";
   write "m:=",mgv,";";

   For ulv:=mgv step -1 until 1 
    do for vlv:=ulv-1 step -1 until 1 do for rlv:=1:mgv do if
     c(ulv,vlv,rlv) neq 0 then write
      "c(",ulv,",",vlv,",",rlv,"):=",c(ulv,vlv,rlv),";";;

   write ";end;";

   shut "$HOME/REDUCE/LIE/PROGS/cnew";

;end;



